package Sophos::LogSearch::Config;
use strict;
use PureMessage::Config qw(expand_filename);
use ActiveState::Config qw(:path);
use File::Path qw(mkpath);
use base qw(Exporter);

our @EXPORT_OK = qw( %config %logsearch_config);
our %config;
our $object;

our $logsearch_config; # alias that doesn't clash with other %configs
*logsearch_config =\%config;


sub os_info {
   my %settings;
   $settings{os} ||= $^O;
   chomp($settings{uname_m} ||= `uname -m` || "unknown");
   chomp($settings{uname_r} ||= `uname -r` || "unknown");
   chomp($settings{uname_v} ||= `uname -v` || "unknown");
   chomp($settings{uname_s} ||= `uname -s` || "unknown");
   return %settings;
}


sub find_maillog {
    my %info = os_info();
    my $os = $info{os};
    
    if ($os eq 'solaris') {
      return '/var/log/syslog';
    }

    # suse 10 logs to /var/log/mail
    foreach my $file (qw(/var/log/maillog /var/log/mail.log /var/log/mail)) {
        return $file if (-e $file && -f _);
    }
    # defaults
    return ($^O eq 'linux') ? '/var/log/mail.log' : '/var/log/maillog';
}

my $pmx_varlog = $PureMessage::Config::config{'log_dir'};
my $message_log = $PureMessage::Config::config{'milter'}->{'Policy'}->{'message_log'} ||  "$pmx_varlog/message_log";

sub d_maillog {
  my $node = shift;
  return find_maillog();
}

sub d_indexes {
  my $default = "$pmx_varlog/search";
  mkpath($default) unless -d $default;
  return $default;
}

sub d_messagelog {
  return $message_log;
}

sub d_logsearch_enabled {
    my $node = shift;

    my $mta = $PureMessage::Config::config{active_mta};

    # We only support postfix with logsearch currently
    if ($mta eq 'postfix') {
        return 1;
    }
    return 0;
}


sub refresh() {

    $object = ActiveState::Config->new(strict => 1, allow_includes => 1);
    Sophos::LogSearch::Config->validate($object);
    $object->parse_file(expand_filename("etc/logsearch.conf"));
    %config = $object->as_hash;
    foreach (qw( active_mta prefix)) {
      $config{$_} = $PureMessage::Config::config{$_};
    }
}

refresh();

=item edit()

 Will open this config as a writable  ActiveState::Config object
 and return it. This can die when configuration file has invalid
 syntax.

=cut

sub edit {
     my $p = ActiveState::Config->new(strict => 1, allow_includes => 1);
     Sophos::LogSearch::Config->validate($p);
     $p->parse_file(expand_filename("etc/logsearch.conf"),writable => 1);
     return $p;
}

sub validate {
    my ($pkg, $p) = @_;

    # Define validation rules and default values.
    my %conf = (
        'path.maillog'            => ['#str', \&d_maillog],
        'path.indexes'            => ['#str', \&d_indexes],
        'path.messagelog'         => ['#str', \&d_messagelog],
        'index_expire_days'       => ['#num', 7],
        'queue_max_items'         => ['#num', 1000],
        'queue_max_time'          => ['#num', 120],
        'remote_search_timeout'   => ['#num', 10],
        'deferred_days'           => ['#num', 2],
        'enabled'                 => ['#bool', \&d_logsearch_enabled],
        'index_local_delivery'    => ['#bool', 1],
        'max_disk_usage'          => ['#size','2GB'],
        'status_max_loops'        => ['#num',600],
    );

    # Add our validation rules to the parser.
    foreach my $key (keys %conf)
    {
        my ($type, $def) = @{$conf{$key}};
        $p->typedecl( $key, $type ) if (defined $type);
        $p->default( $key, $def )   if (defined $def);
    }
}


1;
